unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Generics.Collections;

type
  TfrmMain = class(TForm)
    ListBox1: TListBox;
    PageControl1: TPageControl;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
    FWndCount: Integer;
    FProcesses: TList<DWORD>;
    procedure WMUser1(var Msg: TMessage); message WM_USER + 1;
  public
    { Public-Deklarationen }
    procedure OpenNewTab;

    function AddWindow(AProcessId: DWORD): HWND;
    procedure WindowAdded(Wnd: HWND);
    procedure RemoveWindow(Wnd: HWND);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FProcesses := TList<DWORD>.Create;
  PostMessage(Handle, WM_USER + 1, 0, 0);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
var
  ProcessId: DWORD;
  Process: THandle;
begin
  for ProcessId in FProcesses do
  begin
    Process := OpenProcess(PROCESS_TERMINATE, False, ProcessId);
    if Process <> 0 then
    begin
      TerminateProcess(Process, 0);
      CloseHandle(Process);
    end;
  end;
  FreeAndNil(FProcesses);
end;

procedure TfrmMain.WMUser1(var Msg: TMessage);
begin
  OpenNewTab;
end;

procedure TfrmMain.OpenNewTab;
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  App: string;
begin
  FillChar(StartupInfo, SizeOf(StartupInfo), 0);
  StartupInfo.cb := SizeOf(StartupInfo);
  App := ParamStr(0);

  if CreateProcess(PChar(App), PChar('"' + App + '" -p' + IntToStr(GetCurrentProcessId)), nil, nil, False, 0, nil, nil, StartupInfo, ProcessInfo) then
  begin
    // TODO: Add hProcess to a list that the processs termination monitor thread watches
    // (WaitForMultipleObjects+ResetSignal that is triggered after a process was added/removed)
    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread);
  end
  else
    RaiseLastOSError;
end;

function TfrmMain.AddWindow(AProcessId: DWORD): HWND;
var
  Tab: TTabSheet;
begin
  Tab := TTabSheet.Create(PageControl1);
  Tab.PageControl := PageControl1;
  Tab.Tag := NativeInt(AProcessId);
  Result := Tab.Handle;
end;

procedure TfrmMain.WindowAdded(Wnd: HWND);
var
  ProcessId: DWORD;
  R: TRect;
  I: Integer;
  Len: Integer;
  S: string;
begin
  GetWindowThreadProcessId(Wnd, ProcessId);
  FProcesses.Add(ProcessId);

  for I := 0 to PageControl1.PageCount - 1 do
  begin
    if PageControl1.Pages[I].Tag = NativeInt(ProcessId) then
    begin
      Len := SendMessage(Wnd, WM_GETTEXTLENGTH, 0, 0);
      SetLength(S, Len);
      if Len > 0 then
        SetLength(S, SendMessage(Wnd, WM_GETTEXT, WPARAM(Len + 1), LPARAM(PChar(S))));
      PageControl1.Pages[I].Caption := S;
      Break;
    end;
  end;

  Inc(FWndCount);
  ListBox1.Items.Add('Added: ' + IntToHex(Wnd, 8));

  GetWindowRect(Wnd, R);
  OffsetRect(R, -R.Left, -R.Top);
  Windows.SetWindowPos(Wnd, 0, 0, 0, R.Right, R.Bottom, SWP_NOZORDER or SWP_NOACTIVATE);
end;

procedure TfrmMain.RemoveWindow(Wnd: HWND);
var
  I: Integer;
  ProcessId: DWORD;
begin
  GetWindowThreadProcessId(Wnd, ProcessId);
  FProcesses.Remove(ProcessId);

  for I := 0 to PageControl1.PageCount - 1 do
  begin
    if PageControl1.Pages[I].Tag = NativeInt(ProcessId) then
    begin
      PageControl1.Pages[I].Free;
      Break;
    end;
  end;

  Dec(FWndCount);
  ListBox1.Items.Add('Removed: ' + IntToHex(Wnd, 8));
  if FWndCount = 0 then
    Close;
end;

end.
