program TestProject;

uses
  Windows;

{.$R *.res}

function GetCommunicationHandle(const Name: string): HWND;
var
  FileMappingHandle: THandle;
  P: ^HWND;
begin
  Result := 0;

  FileMappingHandle := OpenFileMapping(FILE_MAP_READ, False, PChar(Name));
  if FileMappingHandle <> 0 then
  begin
    P := MapViewOfFile(FileMappingHandle, FILE_MAP_READ, 0, 0, SizeOf(HWND));
    if P <> nil then
    begin
      Result := P^;
      UnmapViewOfFile(P);
    end;
    CloseHandle(FileMappingHandle);
  end;
end;

procedure ExitLastOSError;
var
  LastError: DWORD;
  Buffer: PChar;
begin
  LastError := GetLastError;
  if LastError <> 0 then
  begin
    FormatMessage(
      FORMAT_MESSAGE_FROM_SYSTEM or
      FORMAT_MESSAGE_IGNORE_INSERTS or
      FORMAT_MESSAGE_ARGUMENT_ARRAY or
      FORMAT_MESSAGE_ALLOCATE_BUFFER,
      nil, LastError, 0, @Buffer, 0, nil);
    MessageBox(0, Buffer, 'TestProject', MB_OK or MB_ICONERROR);
    LocalFree(HLOCAL(Buffer));
    Halt(1);
  end;
end;


var
  Lib: HMODULE;
  CommunicationHandle: HWND;
  InitializeModule: procedure(ACommunicationHandle: HWND);

  I: Integer;
  S: string;
  ProcessId: DWORD;
  ProcessHandle: THandle;
  ProcessIdStr: string;
  Err: Integer;

  ExePath: string;
begin
  ProcessId := 0;
  CommunicationHandle := 0;


  ExePath := ParamStr(0);
  I := Length(ExePath);
  while (I > 0) and (ExePath[I] <> '\') do
    Dec(I);
  ExePath := Copy(ExePath, 1, I);

  for I := 1 to ParamCount do
  begin
    S := ParamStr(I);
    if (Length(S) > 2) and (S[1] = '-') and (S[2] = 'p') then
    begin
      ProcessIdStr := Copy(S, 3, MaxInt);
      Val(ProcessIdStr, ProcessId, Err);
      if Err = 0 then
        Break;
      ProcessIdStr := '';
    end;
  end;

  if ProcessId <> 0 then
  begin
    ProcessHandle := OpenProcess(PROCESS_QUERY_INFORMATION, False, ProcessId);
    if ProcessHandle = 0 then
      ExitLastOSError;

    CommunicationHandle := GetCommunicationHandle('TestProject_Process_' + ProcessIdStr);
  end;


  if CommunicationHandle <> 0 then
    Lib := LoadLibrary(PChar(ExePath + 'TestProjectCore.dll'))
  else
    Lib := LoadLibrary(PChar(ExePath + 'TestProjectMain.dll'));

  if Lib = 0 then
    ExitLastOSError;

  @InitializeModule := GetProcAddress(Lib, 'InitializeModule');
  if not Assigned(InitializeModule) then
    ExitLastOSError;

  InitializeModule(CommunicationHandle);

  FreeLibrary(Lib);
end.
