library TestProjectMain;

uses
  Windows,
  Messages,
  SysUtils,
  Controls,
  Forms,
  Classes,
  JclSysUtils,
  MainFrm in 'MainFrm.pas' {frmMain};

{$R *.res}

const
  CM_ADDTAB = CM_BASE + 1;
  CM_TABADDED = CM_BASE + 2;
  CM_REMOVETAB = CM_BASE + 3;
  CM_OPENTAB = CM_BASE + 4;

type
  PSharedMemComunication = ^TSharedMemComunication;
  TSharedMemComunication = record
    CommunicationHandle: HWND;
  end;

  TCommunicationObject = class(TObject)
  private
    FHandle: HWND;
  protected
    procedure WndProc(var Msg: TMessage);
    procedure CMAddTab(var Msg: TMessage); message CM_ADDTAB;
    procedure CMTabAdded(var Msg: TMessage); message CM_TABADDED;
    procedure CMRemoveTab(var Msg: TMessage); message CM_REMOVETAB;
    procedure CMOpenTag(var Msg: TMessage); message CM_OPENTAB;

    function ReadStringMessage(var Msg: TMessage): string;
  public
    constructor Create;
    destructor Destroy; override;
    procedure DefaultHandler(var Message); override;

    property Handle: HWND read FHandle;
  end;

{ TCommunicationObject }

procedure TCommunicationObject.CMAddTab(var Msg: TMessage);
begin
  Msg.Result := LRESULT(frmMain.AddWindow(DWORD(Msg.WParam)));
end;

procedure TCommunicationObject.CMTabAdded(var Msg: TMessage);
begin
  frmMain.WindowAdded(HWND(Msg.LParam));
end;

procedure TCommunicationObject.CMOpenTag(var Msg: TMessage);
var
  S: string;
begin
  S := ReadStringMessage(Msg);
  frmMain.OpenNewTab;
end;

procedure TCommunicationObject.CMRemoveTab(var Msg: TMessage);
begin
  frmMain.RemoveWindow(HWND(Msg.LParam));
end;

constructor TCommunicationObject.Create;
begin
  inherited Create;
  FHandle := AllocateHWnd(WndProc);
end;

destructor TCommunicationObject.Destroy;
begin
  DestroyWindow(FHandle);
  inherited Destroy;
end;

function TCommunicationObject.ReadStringMessage(var Msg: TMessage): string;
var
  Process: THandle;
  Len: Longint;
  n: DWORD;
begin
  if Pointer(Msg.LParam) <> nil then
  begin
    Len := 0;
    Process := OpenProcess(PROCESS_ALL_ACCESS, False, DWORD(Msg.WParam));
    if Process = 0 then
      RaiseLastOSError;
    if not ReadProcessMemory(Process, Pointer(Msg.LParam - SizeOf(Longint)), @Len, SizeOf(Len), n) then
      RaiseLastOSError;
    SetLength(Result, Len);
    if Len > 0 then
      ReadProcessMemory(Process, Pointer(Msg.LParam), PChar(Result), Len * SizeOf(Char), n);
  end
  else
    Result := '';
end;

procedure TCommunicationObject.DefaultHandler(var Message);
begin
  with TMessage(Message) do
    Result := DefWindowProc(FHandle, Msg, WParam, LParam);
end;

procedure TCommunicationObject.WndProc(var Msg: TMessage);
begin
  Dispatch(Msg);
end;

{ Exports }

procedure InitializeModule(ACommunicationHandle: HWND);
var
  Communication: PSharedMemComunication;
  CommObject: TCommunicationObject;
begin
  CommObject := TCommunicationObject.Create;
  try
    Communication := SharedAllocMem('TestProject_Process_' + IntToStr(GetCurrentProcessId), SizeOf(TSharedMemComunication));
    try
      Communication.CommunicationHandle := CommObject.Handle;

      Application.Initialize;
      Application.ShowMainForm := True;
      Application.Title := 'TestProject';
      Application.CreateForm(TfrmMain, frmMain);
      Application.Run;

    finally
     SharedFreeMem(Communication);
    end;
  finally
    CommObject.Free;
  end;
end;

exports
  InitializeModule;

begin
end.
